; 
;  PALETTE.ASM                                          
; Ĳ
;  Palette management in mode x or mode 13h             
;  (PMODE VERSION)                                      
; Ĳ
;  Made by Daan Nusman, ULTRUM SOFT '94                 
; 

        .386p
        jumps

code32  segment para public use32
        assume cs:code32, ds:code32

include ..\pmode.inc
include ..\file\file.inc

; 
;  PUBS                                                 
; 

PUBLIC overall_palette

PUBLIC loadpalette
PUBLIC shift_palette_left
PUBLIC shift_palette_right
PUBLIC palette_fadeoff
PUBLIC palette_equalize

; 
;  DATA                                                 
; 

   handle          dw ?            ; FILE HANDLE
   overall_palette db 3*256 dup(?)

; 
;  LOADPALETTE ... loads a palette                      
; Ĳ
; In:                                                   
;  EDX -> FileName (ASCIIZ)                             
; 
loadpalette:
        call _openfile                  ; this will go well if you have EDX pointing to the file name
        jc stopit_with_error

        mov ax,v86r_bx                  ; save file handle
        mov handle,ax

        mov edx,offset overall_palette  ; offset palette
        mov ecx, 3*256                  ; size palette (ALWAYS 3*256)
        call _readfile
        jc stopit_with_error

        mov ax,handle           ; close the file...
        mov v86r_bx,ax
        call _closefile

stopit_without_error:
        clc
        ret
stopit_with_error:
        stc
        ret

; 
;  SHIFT_PALETTE_LEFT ... the name says it all... well, 
;    almost, one note: it's a palette in memory, not    
;    the real DAC is being reprogrammed                
; Ĳ
; In:                                                   
;  EDX -> palette (3*256 bytes)                         
;  EAX - First color to modify                          
;  EBX - Last color to modify                           
;  ECX - How many colors to shift left                  
; 
; -- Local variables --
align 4
FCN     dd ?
LCN     dd ?
hmc     dd ?
all_mem dd ?
palptr  dd ?
shift_palette_left:
  push edi esi ebx eax ecx
  mov esi,edx
  mov palptr,edx
  mov eax,3*256
  call _getlomem
  jc emer_exit
  ;EAX -> Allocated block... (I hope)
  mov edi,eax
  mov all_mem,eax
  pop ecx
  ;Copy CX*3 bytes
  mov edx,ecx
  add edx,ecx
  add edx,ecx          ; EDX = 3*ECX
                       ; EDX = how many bytez to loop...
                       ; EDI -> Allocated block of mem...
                       ; ESI -> Palette START..
  pop eax              ; EAX = 1st color number
  mov ecx,eax          
  add ecx,eax
  add ecx,eax          ; ECX 1stcol * 3
  add esi,ecx          ; ESI -> Palette begin...
  mov FCN,ecx
  mov ecx,edx          ; ECX = how many bytes to copy
  mov hmc,ecx

                       ; EDI -> Allocated mem..
  push esi ecx
  rep movsb            ; copy this Bull over to the allocated part of mem...
  pop ecx esi
  ; move from palette + 1stcolor to palettestart .
  ; ESI = palette ptr (+offset)
  ; EDI = palette ptr + how many bytes to copy...
  mov edi,esi
  add edi,ecx
  xchg esi,edi
  pop eax               ; Haal last color number...
  
  mov ecx,eax
  add ecx,eax
  add ecx,eax
  mov LCN,ecx
  sub ecx,FCN           ; ECX = (LCN*3)-(FCN*3) = Number

  rep movsb             ; Shift palette back...
  mov esi,all_mem       ; SOURCE: Allocated memory
  mov edi,palptr        ; DESTINATION: Top of palette, part that hasn't been shifted
  add edi,LCN
  sub edi,hmc
  add edi,3
  mov ecx,hmc
  rep movsb             ; Move saved bytes from temp. area to top of palette
  
  sub d _lomembase,3*256 ; Free up allocated mem
  pop  esi edi
  ret                   ; And Back...
emer_exit:
  pop ecx eax ebx esi edi
  ret

; 
;  SHIFT_PALETTE_RIGHT ... the name says it all...      
; Ĳ
; In:                                                   
;  EDX -> palette (3*256 bytes)                         
;  EAX - First color to modify                          
;  EBX - Last color to modify                           
;  ECX - How many colors to shift right                 
; 
; -- Local variables --
align 4
bytes_to_copy   dd ?
bytes_to_start  dd ?
bytes_to_end    dd ?
allocated_block dd ?
shift_palette_right:
  push esi edi edx
  ; Find out how many bytes to copy...
  push eax
  mov eax,ecx           ; EAX = ECX * 1
  add eax,ecx           ; EAX = ECX * 2
  add eax,ecx           ; EAX = ECX * 3
  mov bytes_to_copy,eax
  pop eax
  ; Find out how many bytes the start is...
  mov ecx, eax
  add ecx, eax
  add ecx, eax
  mov bytes_to_start,ecx
  ; Find out how many bytes the end is
  mov ecx, ebx
  add ecx, ebx
  add ecx, ebx
  mov bytes_to_end,ecx

  ; Okay... everything is calculated... now uhhmm
  ; allocate the memory...
  mov eax, 3*256
  call _getlomem
  jc emergency_exit
  pop edx
  ; eax -> allocated block..
  mov allocated_block,eax
  mov edi, eax
  mov esi, edx            ; EDX was the ptr to the palette, remember
  add esi, bytes_to_end
  push esi                ; 765
  sub esi, bytes_to_copy
  add esi, 3
  mov ecx, bytes_to_copy
  
  rep movsb                ; Copy top of palette to temp buffer
  pop  esi                ; VERIFIED

  ; Dest   = start of block just copied (EDI)
  ; Source = start of block just copied (ESI) - Bytes to copy 
  ; esi = 765 = last color
  mov edi, esi
  sub esi, bytes_to_copy    ; 765 - 6 = 759 = color 253  = 255 - 2 = correct
  inc edi
  inc edi
  inc esi
  inc esi
  mov ecx, bytes_to_end
  sub ecx, bytes_to_start
  std
  rep movsb
  cld
  mov edi,edx              ; EDX is STILL the adress
  add edi, bytes_to_start  ; Put upper part (wich is now shifted
  mov ecx, bytes_to_copy   ; away) of palette from temp area to
  mov esi, allocated_block ; lower part of palette)
  rep movsb                ; Boom.
  sub d _lomembase, 3*256  ; Release memory
  pop  edi esi
  ret                      ; ... That's all, folx
emergency_exit:
  pop edx edi esi
  stc
  ret

; 
;  PALETTE_FADEOFF ... decreases a palette in memory.   
;    this will result in a fade out-effect.             
; Ĳ
; In:                                                   
;  EDX -> palette (3*256 bytes)                         
;  EAX - First color to modify                          
;  EBX - Last color to modify                           
; 
palette_fadeoff:
  push edi ecx
  ; Find out where to start -> EDX
  add edx,eax           ; EDX += EAX*3
  add edx,eax
  add edx,eax
  ; Find out how many colors to decrease -> ECX
  sub ebx,eax
  inc ebx
  mov ecx,ebx
  add ecx,ebx
  add ecx,ebx
align 4
decrease_loop:
  cmp byte ptr [edx],0
  jz skip_decr
  dec byte ptr [edx]
skip_decr:
  inc edx
  loop decrease_loop
  pop  ecx edi
  ret

; 
;  PALETTE_EQUALIZE... Equalizes a palette in memory.   
;    This will result in a fade trough-effect.          
; Ĳ
; In:                                                   
;  EDX -> palette to modify (destination) (3*256 bytes) 
;  ECX -> palette to modify TO (source) (3*256 bytes)   
;  EAX - First color to modify                          
;  EBX - Last color to modify                           
; 
palette_equalize:
  push edi ecx
  ; Find out where to start (DEST) -> EDX
  add edx,eax           ; EDX += EAX*3
  add edx,eax
  add edx,eax
  ; Find out where to start (SRC) -> ECX, EDI
  add ecx,eax           ; ECX += EAX*3
  add ecx,eax
  add ecx,eax
  mov edi,ecx
  ; Find out how many colors to equalize -> ECX
  sub ebx,eax           ; Last-First+1
  inc ebx
  mov ecx,ebx           ; (Last-First+1)*3
  add ecx,ebx
  add ecx,ebx
align 4
equalize_loop:
  mov al, byte ptr [edx]
  cmp byte ptr [edi], al
  je skip_eq
  jl dec_dest  ;is SRC smaller than DEST? 
  inc byte ptr [edx]    ; No, increase...
  jmp skip_eq
dec_dest:
  dec byte ptr [edx]    ; Yea, decrease...
skip_eq:
  inc edx
  inc edi
  loop equalize_loop
  pop  ecx edi
  ret


code32  ends
        end

